@@ -1,5 +1,5 @@
-#kubik: Cubic Hermite Splines
-#Copyright (C), Abby Spurdle, 2019
+#kubik: Cubic Hermite Splines and Related Optimization Methods
+#Copyright (C), Abby Spurdle, 2020
 
 #This program is distributed without any warranty.
 
@@ -11,33 +11,40 @@
 #Also, this license should be available at:
 #https://cran.r-project.org/web/licenses/GPL-2
 
-print.k.spline = function (x, ...)
-	object.summary (x, ...)
+print.kspline = function (x, ...)
+{	if ("package:intoo" %in% search () )
+	{	str = "object.summary (x, ...)"
+		eval (str2lang (str) )
+	}
+	else
+		print.default (x, ...)
+}
 
-plot.k.spline = function (x, with.points=FALSE, ..., pch=16)
+plot.kspline = function (x, ..., control.points=FALSE)
 {	f = x
 
 	. = attributes (f)
 	x = seq (.$cx [1], .$cx [.$nc], length.out=200)
 	y = f (x)
 	plot (x, y, type="l", ...)
-	if (with.points)
-		points (.$cx, f (.$cx), pch=pch)
+	if (control.points)
+		points (f, ...)
 }
 
-lines.k.spline = function (x, with.points=FALSE, ..., pch=16)
+lines.kspline = function (x, ..., control.points=FALSE)
 {	f = x
 
 	. = attributes (f)
 	x = seq (.$cx [1], .$cx [.$nc], length.out=200)
 	lines (x, f (x), ...)
-	if (with.points)
-		points (.$cx, f (.$cx), pch=pch, ...)
+	if (control.points)
+		points (f, ...)
 }
 
-points.k.spline = function (x, ..., pch=16)
-{	f = x
-
-	. = attributes (f)
+.points.kspline = function (f, ..., pch=16)
+{	. = attributes (f)
 	points (.$cx, f (.$cx), pch=pch, ...)
 }
+
+points.kspline = function (x, ...)
+	.points.kspline (x, ...)
